/**
 * Parse the commandline parameters given to deskapp,
 * and execute the corresponding actions.
 * @param {Array} argv containing the commandline parameters.
 */
function _parseAndProcessArgv(argv) {
	/* Create a new parser */
	const getopt = require('node-getopt').create([
	  ['' , 'create'      , 'Create a new e-mail.'],
	  ['' , 'contact'     , 'Create a new contact.'],
	  ['' , 'task'        , 'Create a new task.'],
	  ['' , 'appointment' , 'Create a new appointment.'],
	  ['' , 'meeting'     , 'Create a new meeting.'],
	  ['' , 'note'        , 'Create a new note.'],
	  ['' , 'mailto=ARG'  , 'Use a mailto URL.'],
	  ['' , 'durl[=ARG]'  , 'This is for Linux .desktop'],
	  ['' , 'attach=ARG+' , 'Add attachment.'],
	  ['' , 'subject=ARG' , 'Add subject.'],
	  ['' , 'to=ARG+'     , 'Recipients. Multiple definitions possible.'],
	  ['' , 'cc=ARG+'     , 'CC. Multiple definitions possible.'],
	  ['' , 'bcc=ARG+'    , 'BCC. Multiple definitions possible.'],
	  ['' , 'body=ARG'    , 'Add mail body.'],
	  ['' , 'eml=ARG'     , 'Open an eml file'],
	  /* 
	   * Dummies for cmdline from open event.
	   * They are automatically added on windows systems, so catch them 
	   * to not throw an error.
	   */
	  ['' , 'js-flags'                       , 'Catch it'],
	  ['' , 'original-process-start-time=ARG', 'Catch it'],
	  ['' , 'remember-cert-error-decisions ' , 'Catch it'],
	  ['' , 'ignore-certificate-errors '     , 'Catch it'],
	  ['' , 'ignore-urlfetcher-cert-requests', 'Catch it'],
	  ['' , 'enable-spell-checking'          , 'Catch it'],
	  ['' , 'auth-server-whitelist'          , 'Catch it'],
	  ['' , 'importini'                      , 'Catch it']
	]);

	/* Register actions */
	getopt.on("create", function(argv, options) {
		if (options.mailto) {
			return;
		}
		global.DA.winWebApp.createMail(options);
	});
	getopt.on("contact", function(argv, options) {
		global.DA.winWebApp.newItem('contact');
	});
	getopt.on("task", function(argv, options) {
		global.DA.winWebApp.newItem('task');
	});
	getopt.on("appointment", function(argv, options) {
		global.DA.winWebApp.newItem('appointment');
	});
	getopt.on("meeting", function(argv, options) {
		global.DA.winWebApp.newItem('meeting');
	});
	getopt.on("note", function(argv, options) {
		global.DA.winWebApp.newItem('note');
	});
	getopt.on("mailto", function(argv, options) {
		_parseMailto(options);
	});
	getopt.on("durl", function(argv, options) {
		_parseURL(options);
	});
	getopt.on("eml", function(argv, options) {
		_openEML(options);
	});


	/* Do not kill the program if an error in parsing occurs. */
	getopt.error(function(e) {console.log("Error parsing argv:" + e);});

	/* Parse the commandline and execute corresponding actions */
	getopt.parse(argv);

	return;
}

/**
 * Parse a mailto url to a getopt options object, the call createMail
 * @param {Object} options getopt options object containing the mailto url
 *	and receiving the converted option items.
 */
function _parseMailto(options) {
	var atr, isMultiValue;
	var mailtoParser = require("mailto-parser");
	var parsedMailto = mailtoParser.parse(options.mailto, true);
	var addresses = parsedMailto.addresses;
	var attributeKey = parsedMailto.attributeKey;
	var attributeKeys = ['cc', 'bcc', 'subject', 'body', 'attach'];

	/* Libre Office uses attachment instead of attach */
	if (typeof attributeKey['attachment'] !== 'undefined') {
		if (typeof attributeKey['attach'] === 'undefined') {
			attributeKey['attach'] = attributeKey['attachment'];
		} else {
			attributeKey['attach'] = attributeKey['attach']+ ',' + (attributeKey['attachment']);
		}
	}

	/* Add addresses */
	if (addresses.length > 0) {
		if (typeof options['to'] === 'undefined') {
			options['to'] = [];
		}
		for (var i = 0; i < addresses.length; i++) {
			options['to'].push(decodeURIComponent(addresses[i]));
		}
	}

	/* Add attributeKeys */
	for (var i = 0; i < attributeKeys.length; i++) {
		atr = attributeKeys[i];
		isMultiValue = (atr === 'cc' || atr === 'bcc' || atr === 'attach');
		/* Skip if not defined */
		if (typeof attributeKey[atr] === 'undefined') {
			continue;
		}
		if (isMultiValue) {
			if (typeof options[atr] === 'undefined') {
				options[atr] = [];
			}
			// Split by , before decoding
			var encoded = attributeKey[atr].split(',');
			for (var j = 0; j < encoded.length; j++) {
				options[atr].push(decodeURIComponent(encoded[j]));
			}
		} else {
			options[atr] = decodeURIComponent(attributeKey[atr]);
		}
	}

	global.DA.winWebApp.createMail(options);
	return;
}

/**
 * This is needed because of the differences between argv and cmdline.
 * argv is vailable in the first instance of DeskApp and cmdline from
 * the following. argv is an array, cmdline a string. But getopt needs
 * an array for parsing.
 * The cmdline string also differs between windows and linux.
 * @param {String} cmdline the commandline as a String
 * @return {Array} commandline parsed to an array
 */

function _cmdlineToArgv(cmdline) {
	var regex = /--[^\s]+="[^"]+"|--[^\s"]*/g;
	var cmdArr = [];
	var match;
	do {
		match = regex.exec(cmdline);
		if (match !== null) {
			cmdArr.push(match[0].replace(/"/g, ''));
		}
	} while (match !== null);
	return cmdArr;
}

/**
 * This is needed for sendto by nautilus, because it forms incorrect mailto urls.
 * "mailto:?attach=file:///path/to/file1&attach=file:///path/to/file2"
 * so move them to options.attachment array, after this let parseMailto do the rest.
 * @param {Object} options options object from getopt
 */
function _parseURL(options) {
	if (!options.durl) {
		return;
	}

	var sUrlTmp = options.durl;
	var regex = /[&]*attach=([^&=]*)/g;
	var match;

	do {
		match = regex.exec(sUrlTmp);
		if (match !== null) {
			if (typeof options['attach'] === 'undefined') {
				options['attach'] = [];
			}
			options.durl = options.durl.replace(match[0], '');
			options.attach.push(match[1]);
		}
	} while (match !== null);

	options.mailto = options.durl;
	_parseMailto(options);
	return;
}

function _openEML(options) {
	console.log("_openEML: " + options.eml);
	global.DA.winWebApp.openEML(options.eml.replace(/"/g,''));
}

module.exports = {
	parseAndProcessArgv: _parseAndProcessArgv,
	cmdlineToArgv: _cmdlineToArgv,
	parseMailto: _parseMailto
};

